(function ($) {
  Drupal.theme.itsContentChangedWarning = function () {
    return '<div class="messages warning">' + Drupal.theme('tableDragChangedMarker') + ' ' + Drupal.t('The changes to the content will not be saved until the <em>Save</em> button is clicked.') + '</div>';
  };

  Drupal.its = {
    settings:{}
  };

  Drupal.its.filter = {

    colors:[
      ['#444444', '#dddddd'],
      ['#880000', '#dddddd'],
      ['#008800', '#dddddd'],
      ['#000088', '#dddddd'],
      ['#008888', '#dddddd']
    ],
    active_filters:{},

    /**
     * Return whether the passed node belongs to this plugin.
     *
     * @param node
     *   The currently focused DOM element in the editor content.
     * @param meta_name
     *   Name of the meta_data to process
     * @param selector
     *   Add if there is a jQuery ITS parser selector, how it could be checked if this is active
     */
    isNode:function (node, meta_name, selector) {
      if (selector) {
        var el = (node.jquery) ? node : $(node);
        return el.is(selector);
      }
      if (!(meta_name in Drupal.its.settings) || !('validAttributes' in Drupal.its.settings[meta_name])) {
        return false;
      }
      var hasAtt = false,
        el = (node.jquery) ? node : $(node);
      $.each(Drupal.its.settings[meta_name].validAttributes, function (key, value) {
        if (el.is('[' + value + ']')) {
          hasAtt = true;
        }
      });
      return hasAtt;
    },

    /**
     * Highlight a context and add a tooltip.
     *
     * @param meta_name
     * @param context
     * @param selector
     */
    show:function (meta_name, context, selector) {
      var filter = this;
      if (!(meta_name in this.active_filters)) {
        var color = this.colors.shift();
        this.active_filters[meta_name] = {color:color};
        $('a[href="#' + meta_name + '"]').css('background-color', color[0]).css('color', color[1]);
      }

      if (meta_name === 'its_translate' || meta_name === 'its_locNote') {
        if (context.jquery && context.is(selector)) {
          filter.addHighlightCSS(meta_name, context);
        }
        $(selector, context).each(function(key, element) {
          filter.addHighlightCSS(meta_name, element);
        });
      }
      else {
        if (filter.isNode(context, meta_name, selector)) {
          filter.addHighlightCSS(meta_name, context);
        }
        // check child nodes
        $.each($(context).children(), function (key, value) {
          filter.show(meta_name, value, selector)
        });
      }
    },

    addHighlightCSS: function(meta_name, element) {
      if (meta_name in this.active_filters) {
        var color = this.active_filters[meta_name].color;
        var $element = (element.jquery) ? element : $(element);
        $element.css('background-color', color[0]).css('color', color[1]);
        this.refreshToolTip($element);
      }
    },

    removeHighlightCSS: function(meta_name, element) {
      if (meta_name in this.active_filters) {
        var color = this.active_filters[meta_name].color;
        var $element = (element.jquery) ? element : $(element);
        if (this.rgb2hex($element.css('background-color')) == color[0]) {
          $element.css('background-color', '');
        }
        if (this.rgb2hex($element.css('color')) == color[1]) {
          $element.css('color', '');
        }
        // If other filters are active, use new style.
        var filter = this;
        $.each(this.active_filters, function(type, data) {
          if (type == meta_name) {
            return true;
          }
          var checkbox = $('.meta-data input.form-checkbox[value=' + type + ']'),
            selector = null;
          var data_checkbox = checkbox.data();
          if (data_checkbox && data_checkbox.selector) {
            selector = data_checkbox.selector;
          }
          if (filter.isNode($element, type, selector)) {
            filter.addHighlightCSS(type, $element);
          }
        });
        if ($element.attr('style') === '') {
          $element.removeAttr('style');
        }
        this.refreshToolTip($element, meta_name);
      }
    },

    /**
     * Refresh the tooltip of the selected element.
     * @param $element
     * @param remove name of a data category to remove
     */
    refreshToolTip: function($element, remove) {
      var active_meta = [];
      $.each(this.active_filters, function(meta_name, color) {
        active_meta.push(meta_name);
      });
      var index = active_meta.indexOf(remove);
      if (index > -1) {
        delete active_meta[index];
      }
      var tooltip = '';
      var itsData = $element.getITSData();

      if (active_meta.indexOf('its_translate') > -1 && !itsData.translate) {
        if (tooltip !== '') {
          tooltip += '<br/>';
        }
        tooltip += Drupal.t('This part will not be translated');
      }
      if (active_meta.indexOf('its_locNote') > -1 && itsData.locNote) {
        if (tooltip !== '') {
          tooltip += '<br/>';
        }
        tooltip += 'locNote=' + itsData.locNote;
        if (itsData.locNoteType) {
          tooltip += '<br/>';
          tooltip += 'locNoteType=' + itsData.locNoteType;
        }
      }

      // TODO: add this to the jQuery Plugin
      if (active_meta.indexOf('its_locale') > -1 && $element.attr('its-locale-filter-list')) {
        if (tooltip !== '') {
          tooltip += '<br/>';
        }
        tooltip += 'localeFilterList=' + $element.attr('its-locale-filter-list');
      }
      // TODO: add this to the jQuery Plugin
      if (active_meta.indexOf('its_lang') > -1 && $element.attr('lang')) {
        if (tooltip !== '') {
          tooltip += '<br/>';
        }
        tooltip += 'lang=' + $element.attr('lang');
      }

      var simpletip = $element.data('simpletip');
      if (tooltip !== '') {
        if (typeof simpletip == 'undefined') {
          var pos = $element.position(),
            elOffset = $element.offset(),
            offset = [pos.left - elOffset.left + 5, pos.top - elOffset.top + 5];
          $element.simpletip({content: tooltip, fixed: true, position: 'bottom', offset: offset});
          simpletip = $element.data('simpletip');
          simpletip.getTooltip().appendTo('#mlw-lt-tooltip-container');
        }
        else {
          simpletip.update(tooltip);
        }
      }
      else {
        if (simpletip) {
          simpletip.getTooltip().remove();
          $element.removeData('simpletip');
        }
      }
    },

    /**
     * Remove the color highlight of the elements.
     *
     * @param meta_name
     * @param context
     * @param selector
     */
    hide:function (meta_name, context, selector) {
      var filter = this;
      if (!(meta_name in this.active_filters)) {
        return;
      }

      if (meta_name === 'its_translate' || meta_name === 'its_locNote') {
        if (context.jquery && context.is(selector)) {
          filter.removeHighlightCSS(meta_name, context);
        }
        $(selector, context).each(function(key, element) {
          filter.removeHighlightCSS(meta_name, element);
        });
      }
      else {
        if (filter.isNode(context, meta_name, selector)) {
          filter.removeHighlightCSS(meta_name, context);
        }
        // check child nodes
        $.each($(context).children(), function (key, value) {
          filter.hide(meta_name, value, selector)
        });
      }
    },

    /**
     * Function to convert a rgb format to a hex color.
     *
     * @param rgb string with rgb color code
     * @return {String} hex color code
     */
    rgb2hex:function (rgb) {
      if (typeof rgb == 'undefined' ||  rgb == null || rgb == 'transparent') {
        return "";
      }
      rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
      if (!rgb) {
        return "";
      }
      return "#" +
        ("0" + parseInt(rgb[1], 10).toString(16)).slice(-2) +
        ("0" + parseInt(rgb[2], 10).toString(16)).slice(-2) +
        ("0" + parseInt(rgb[3], 10).toString(16)).slice(-2);
    },

    /**
     * Add meta data to the selected content.
     *
     * @param event
     * @param meta_name
     */
    add:function (event, meta_name) {
      // IE9+, anything else
      // Retrieving a dom element from a selection is a pain in the ass!
      if (window.getSelection) {
        var selection = window.getSelection();

        if (selection.rangeCount) {
          var range = selection.getRangeAt(0);
          // its only allowed to add this to fields
          if ($(range.startContainer).parents('.field-mlw-lt-edit-text').length == 0) {
            return;
          }
          if (range.collapsed) {
            var currentNode = range.startContainer;
            while (currentNode.nodeName === '#text' && currentNode.parentNode) {
              currentNode = currentNode.parentNode;
            }
            if (currentNode.nodeName === '#text') {
              var outerSpan = $(document.createElement('span'));
              outerSpan.html(currentNode);
              currentNode = outerSpan.get(0);
              range.insertNode(currentNode);
              this.defineITSData(meta_name, outerSpan);
            }
            else {
              this.defineITSData(meta_name, $(currentNode));
            }
          }
          else {
            var content,
              startNode = range.startContainer.childNodes[range.startOffset] || range.startContainer,
              endNode   = range.endContainer.childNodes[range.endOffset] || range.endContainer;
            console.log(startNode, range.startOffset);
            console.log(endNode, range.endOffset);

            console.log(startNode.nodeName, endNode.nodeName);
            // selection of a text
            if (startNode.nodeName === '#text' && endNode.nodeName === '#text') {
              // TODO: detect switching of nodes <p>|aaaa</p><p>bbbb|</p>.

              // The selected text has the same length then the node which is selected.
              // Therefore we selected this node completely and don't add an additional span.
              // For Chrome (selection is inside of the node - <a>    |aaaa|</a> ).
              if (startNode.parentNode && range.startOffset == 0 && $.trim(range.toString()) === $.trim(startNode.parentNode.textContent)) {
                content = $(startNode.parentNode);
              }
              // For Firefox (selection is outside of the node - |<a>    aaaa</a>| ).
              else if (startNode.nextSibling && range.endOffset == 0 && $.trim(range.toString()) === $.trim(startNode.nextSibling.textContent)) {
                content = $(startNode.nextSibling);
              }
              // Not the complete text is selected, so add a span around the selected text now.
              else {
                content = $(document.createElement('span'));
                content.html(range.extractContents());
                range.insertNode(content.get(0));
              }
            }
            // Paragraph or similar selected.
            else {
              // TODO: Handel selection of complete paragraph.
              console.log('TODO', range.commonAncestorContainer);
            }

            if (typeof content !== 'undefined') {
              this.defineITSData(meta_name, content);
            }
          }
        }
      }
    },

    /**
     * Depending on type open a modal to input the ITS data and set this data or set it directly (only translate).
     *
     * @param type string
     *   One of translate|locNote|stroageSize|allowedCharacters.
     *
     * @param $element
     *   jQuery element which is selected.
     */
    defineITSData: function(type, $element) {
      var modal = false,
        edit = false,
        global = true,
        itsData = $element.getITSData();

      switch (type) {
        case 'its_translate':
        default:
          if (!itsData.translate) {
            // No global rules.
            if ($element.get(0).hasAttribute('translate')) {
              $element.removeAttr('translate');
            }
            // This is set by a global rule, so force to translate.
            else {
              // FF, IE, ..
              if (typeof $element.get(0).translate === 'undefined') {
                $element.attr('translate', 'yes');
              }
              // Chrome
              else {
                $element.attr('translate', true);
              }
            }
          }
          else {
            // Its translate='yes', so it seems there is a global rule, for it, so only remove this attribute.
            if ($element.get(0).hasAttribute('translate') && ($element.attr('translate') === 'yes' || $element.attr('translate') === true)) {
              $element.removeAttr('translate');
            }
            // No global rules.
            else {
              // FF, IE, ..
              if (typeof $element.get(0).translate === 'undefined') {
                $element.attr('translate', 'no');
              }
              // Chrome
              else {
                $element.attr('translate', false);
              }
            }
          }
          this.finishEditing(false, type, $element);
          break;

        case 'its_locNote':
          modal = $('<div id="edit-form" title="Localization Note">' +
            '<label for="edit-form-locNoteType">Type</label><select id="edit-form-locNoteType" class="value"><option value="description">description</option><option value="alert">alert</option></select><br>' +
            '<label for="edit-form-locNote">List</label><textarea id="edit-form-locNote" class="value"></textarea>' +
            '</div>');
          if (itsData.locNote) {
            edit = true;
            if ($element.attr('its-loc-note')) {
              global = false;
            }
          }
          break;

        case 'its_locale':
          modal = $('<div id="edit-form" title="Locale Filter">' +
            '<label for="edit-form-localeFilterList">Filter</label><input id="edit-form-localeFilterList" class="value" type="value" /><br>' +
            'Comma separated list of languages this should be included.' +
            '</div>');
          if ($element.attr('its-locale-filter-list')) {
            edit = true;
            global = false;
            itsData.localeFilterList = $element.attr('its-locale-filter-list');
          }
          break;

        case 'its_lang':
          modal = $('<div id="edit-form" title="Language">' +
            '<label for="edit-form-lang">Language</label><input id="edit-form-lang" class="value" type="value" /><br>' +
            'Define in which language the selected content is.' +
            '</div>');
          if ($element.attr('lang')) {
            edit = true;
            global = false;
            itsData.lang = $element.attr('lang');
          }
          break;

        // TODO: rename to text Analysis and get it to work (after finalization from W3C)
        case 'its_lexicalizes':
          alert('Not yet finalized by W3C.');
          break;

      }

      if (modal) {
        modal.hide();
        $('body').append(modal);
        modal.dialog({
          autoOpen:false,
          modal:true
        });

        $('.value', modal).each(function() {
          $(this).val('');
        });
        $.each(itsData, function(key, value) {
          var inputField = $('#edit-form-' + key, modal);
          if (inputField.length != 0) {
            inputField.val(value);
          }
        });

        var buttons = {},
          filter = this;
        if (edit) {
          buttons.edit = function () {
            $.each(filter.getDataFromModal(modal), function(attrName, value) {
              $element.attr(attrName, value);
            });
            filter.finishEditing(this, type, $element);
          };
          if (!global) {
            buttons.remove = function () {
              $.each(filter.getDataFromModal(modal), function(attrName, value) {
                $element.removeAttr(attrName);
              });
              filter.finishEditing(this, type, $element);
            }
          }
        }
        else {
          buttons.add = function () {
            $.each(filter.getDataFromModal(modal), function(attrName, value) {
              $element.attr(attrName, value);
            });
            filter.finishEditing(this, type, $element);
          };
        }
        buttons.close = function () {
          filter.finishEditing(this, type, $element);
        };

        modal.dialog('option', 'buttons', buttons);
        modal.dialog('open');

      }
    },

    /**
     * Standard function after editing a element.
     *
     * Close modal and remove it. Update the highlight class and tooltip of the element.
     * Remove empty spans.
     *
     * @param modal
     * @param type
     * @param $element
     */
    finishEditing: function finishEditing(modal, type, $element) {
      if (modal) {
        $(modal).dialog('close').remove();
      }

      // Refresh the highlight of this element, if the checkbox is checked
      var checkbox = $('.meta-data input.form-checkbox[value=' + type + ']');
      if (checkbox.is(':checked')) {
        var data = checkbox.data();
        var selector = null;
        if (data && data.selector) {
          selector = data.selector;
        }
        this.removeHighlightCSS(type, $element);
        if (this.isNode($element, type, selector)) {
          this.show(type, $element, selector);
        }
      }

      // Remove empty spans
      if ($element.get(0).nodeName.toLowerCase() == 'span' && $element.get(0).attributes.length == 0) {
        $element.replaceWith($element.html());
      }

      if ($('#edit-metadata + .messages.warning').length == 0) {
        $(Drupal.theme('itsContentChangedWarning')).insertAfter('#edit-metadata').hide().fadeIn('slow');
      }
    },

    /**
     * Get the input data from a open modal.
     *
     * @param modal
     *
     * @return {Object}
     *   key is the its attribute name and value the input data
     */
    getDataFromModal: function (modal) {
      var data = {};
      $('.value', modal).each(function() {
        var $this = $(this),
          value = $this.val(),
          id = $this.attr('id');
        id = id.replace(/edit-form-/, '');
        id = id.replace(/([A-Z])/g, '-$1').toLowerCase();
        if (id !== 'lang') {
          id = 'its-' + id;
        }
        data[id] = value;
      });
      return data;
    }
  };

  Drupal.behaviors.its_text = {
    attach:function (context, settings) {
      var that = this;

      if ('its' in settings) {
        Drupal.its.settings = settings.its;
      }
      $.parseITS(function(rulesController) {
        $.each(rulesController.supportedRules, function(key, rule) {
          if (rule.NAME === 'translate') {
            that.translateRuleController = rule;
            $('#mlw-lt-translate-values').find('.form-wrapper').each(that.setGlobalTranslateRule);
          }
        });
      });

      $('.meta-data input.form-checkbox:not(.mlw-lt-processed)', context).each(function () {
        $(this).addClass('mlw-lt-processed');
        var data = $(this).data();
        var selector = null;
        if (data && data.selector) {
          selector = data.selector;
        }
        $(this).click(function () {
          that.toggleFilter(this, context, selector);
        });
        that.toggleFilter(this, context, selector);
        $('.field-item a', context).click(function (event) {
          event.preventDefault();
        });
        $(this).next().click(function (event) {
          event.stopPropagation();
          event.preventDefault();
          var meta_name = $(this).children('a').attr('href').substring(1);
          Drupal.its.filter.add(event, meta_name);
        });
      });

      $('#mlw-lt-lm-form').submit(function() {
        // Remove all the highlighting markup.
        $('.meta-data input.form-checkbox:checked', context).each(function () {
          var $this = $(this),
            data = $this.data(),
            selector = null,
            meta_name = $this.val();
          if (data && data.selector) {
            selector = data.selector;
          }
          $('.field-mlw-lt-edit-text', context).each(function () {
            Drupal.its.filter.hide(meta_name, this, selector);
          });
          $('a[href="#' + meta_name + '"]').css('background-color', '').css('color', '');
        });
        // Set the changed input to the textareas.
        $('.field-mlw-lt-edit-text', context).each(function(key, element) {
          var $element = $(element),
            id = $element.parent().attr('id');
          id = '#' + id.substr(0, id.indexOf("-view"));
          $(id).text($element.html());
        });

        return true;
      });

      that.metadataFieldset = $('#edit-metadata');
      that.metadataPosition = that.metadataFieldset.position();
      that.metadataFieldsetWidth = that.metadataFieldset.width();
      that.metadataPosition.top += that.metadataFieldset.height() + 20;
      if (window.parent.Drupal.toolbar) {
        that.metadataFieldsetTop = window.parent.Drupal.toolbar.height();
      }

      $('fieldset.collapsible').bind('collapsed', function() {
        var fieldset = $(this),
          oldHeight = fieldset.height();
        setTimeout(function() {
          var height = fieldset.height();
          if (height > 0) {
            that.metadataPosition.top += height;
          }
          else {
            that.metadataPosition.top -= oldHeight;
          }

          if (window.parent.Drupal.toolbar) {
            that.metadataFieldsetTop = window.parent.Drupal.toolbar.height();
          }
          that.refreshStickTable();
        }, 500)
      });

      $(window).scroll(this.refreshStickTable);
    },

    metadataFieldset: null,
    metadataPosition: {},
    stickyMetaDataFieldSet: false,
    metadataFieldsetWidth: 0,
    metadataFieldsetTop: 10,

    refreshStickTable: function() {
      var self = Drupal.behaviors.its_text;
      if ($(window).scrollTop() > self.metadataPosition.top) {
        if (!self.stickyMetaDataFieldSet) {
          self.stickyMetaDataFieldSet = true;
          self.metadataFieldset.css({position: 'fixed', top: self.metadataFieldsetTop, 'background-color': 'white', width: self.metadataFieldsetWidth});
        }
      }
      else if (self.stickyMetaDataFieldSet) {
        self.stickyMetaDataFieldSet = false;
        self.metadataFieldset.removeAttr('style');
      }
    },

    toggleFilter: function (checkbox, context, selector) {
      if ($(checkbox).is(':checked')) {
        $('.field-mlw-lt-edit-text', context).each(function () {
          Drupal.its.filter.show($(checkbox).val(), this, selector);
        });
      }
      else {
        var meta_name = $(checkbox).val();
        $('.field-mlw-lt-edit-text', context).each(function () {
          Drupal.its.filter.hide(meta_name, this, selector);
        });

        if (meta_name in Drupal.its.filter.active_filters) {
          $('a[href="#' + meta_name + '"]').css('background-color', '').css('color', '');

          Drupal.its.filter.colors.push(Drupal.its.filter.active_filters[meta_name].color);
          delete Drupal.its.filter.active_filters[meta_name];
        }
      }
    },

    translateRuleController: null,
    translateRuleKeyMap: {},

    setGlobalTranslateRule: function(index, fieldset) {
      var self = Drupal.behaviors.its_text;
      var $fieldset    = fieldset.jquery ? fieldset : $(fieldset),
        $selectorInput = $fieldset.find('#edit-mlw-lt-translate-de-' + index + '-mlw-lt-selector'),
        $valueSelect   = $fieldset.find('#edit-mlw-lt-translate-de-' + index + '-mlw-lt-value');

      if (!$selectorInput.hasClass('mlw-lt-global-processed')) {
        $selectorInput.addClass('mlw-lt-global-processed');
        $selectorInput.change(function() {
          self.setGlobalTranslateRule(index, $fieldset);
        });
      }

      if (!$valueSelect.hasClass('mlw-lt-global-processed')) {
        $valueSelect.addClass('mlw-lt-global-processed');
        $valueSelect.change(function() {
          self.setGlobalTranslateRule(index, $fieldset);
        });
      }

      if ($selectorInput.length > 0 && $valueSelect.length > 0 && $selectorInput.val() != '' && $valueSelect.val() != '') {
        var addRuleObject = {
          selector: $selectorInput.val(),
          translate: ($valueSelect.val() === 'yes') ? true : false,
          type: self.translateRuleController.NAME
        };

        if (typeof self.translateRuleKeyMap[index] === 'undefined') {
          self.translateRuleKeyMap[index] = self.translateRuleController.rules.length;
          self.translateRuleController.addSelector(addRuleObject);
          console.log('added', addRuleObject);
        }
        else {
          self.translateRuleController.rules[self.translateRuleKeyMap[index]] = addRuleObject;
          console.log('updated', addRuleObject);
        }
      }
    }
  };

})(jQuery);
