<?php

/**
 * @file
 * WYSIWYG plugin file for MultilingualWeb-LT module.
 * Defines a new Button for WYSIWYG Editors,
 * which allows to add MLW-LT attributes.
 */

/**
 * Define a Wysiwyg plugin, which allows to add MLW-LT attributes.
 *
 * @return array
 *   Meta information about the buttons provided by this plugin.
 */
function its_its_plugin() {

  $plugins['its_translate'] = array(
    // The plugin's title; defaulting to its internal name ('its').
    'title' => t('MultilingualWeb-LT Translate'),
    // The (vendor) homepage of this plugin; defaults to ''.
    'vendor url' => 'http://drupal.org/project/its',
    // The button image filename; defaults to '[plugin-name].png'.
    'icon file' => 'translate.png',
    // The button title to display on hover.
    'icon title' => t('Do not translate. (translate)'),
    // An array of settings for this button. Required, but API is still in flux.
    'settings' => array(
      'validAttributes' => array('translate'),
    ),
    'buttons' => array('its_translate' => t('translate')),
    'load' => TRUE,
    'internal' => FALSE,
    // The path to the button's icon; defaults to
    // '/[path-to-module]/[plugins-directory]/[plugin-name]/images'.
    'icon path' => drupal_get_path('module', 'its') . '/plugins/images',
    // An alternative path to the integration JavaScript; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'js path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration JavaScript; defaults to
    // '[plugin-name].js'.
    'js file' => 'its.wysiwyg.js',
    // An alternative path to the integration stylesheet; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'css path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration stylesheet; defaults to
    // '[plugin-name].css'.
    'css file' => 'its.wysiwyg.css',
  );

  $plugins['its_locale'] = array(
    // The plugin's title; defaulting to its internal name ('its').
    'title' => t('MultilingualWeb-LT Locale Filter'),
    // The (vendor) homepage of this plugin; defaults to ''.
    'vendor url' => 'http://drupal.org/project/its',
    // The button image filename; defaults to '[plugin-name].png'.
    'icon file' => 'locale.png',
    // The button title to display on hover.
    'icon title' => t('Only translate for the following locales. (locale filter)'),
    // An array of settings for this button. Required, but API is still in flux.
    'settings' => array(
      'validAttributes' => array('its-locale-filter-list'),
    ),
    'buttons' => array('its_locale' => t('locale filter')),
    'load' => TRUE,
    'internal' => FALSE,
    // The path to the button's icon; defaults to
    // '/[path-to-module]/[plugins-directory]/[plugin-name]/images'.
    'icon path' => drupal_get_path('module', 'its') . '/plugins/images',
    // An alternative path to the integration JavaScript; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'js path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration JavaScript; defaults to
    // '[plugin-name].js'.
    'js file' => 'its.wysiwyg.js',
    // An alternative path to the integration stylesheet; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'css path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration stylesheet; defaults to
    // '[plugin-name].css'.
    'css file' => 'its.wysiwyg.css',
  );

  // TODO: change to text analysis (waiting till W3C finalized it).
  $plugins['its_lexicalizes'] = array(
    // The plugin's title; defaulting to its internal name ('its').
    'title' => t('MultilingualWeb-LT Text Analysis'),
    // The (vendor) homepage of this plugin; defaults to ''.
    'vendor url' => 'http://drupal.org/project/its',
    // The button image filename; defaults to '[plugin-name].png'.
    'icon file' => 'entity.png',
    // The button title to display on hover.
    'icon title' => t('Specify Concepts that may require special handling for translation. (Text Analysis)'),
    // An array of settings for this button. Required, but API is still in flux.
    'settings' => array(
      'validAttributes' => array(
        'its-disambig-ident-ref',
        'its-disambig-granularity',
        'its-disambig-class-ref'),
    ),
    'buttons' => array('its_lexicalizes' => t('Text Analysis')),
    'load' => TRUE,
    'internal' => FALSE,
    // The path to the button's icon; defaults to
    // '/[path-to-module]/[plugins-directory]/[plugin-name]/images'.
    'icon path' => drupal_get_path('module', 'its') . '/plugins/images',
    // An alternative path to the integration JavaScript; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'js path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration JavaScript; defaults to
    // '[plugin-name].js'.
    'js file' => 'its.wysiwyg.js',
    // An alternative path to the integration stylesheet; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'css path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration stylesheet; defaults to
    // '[plugin-name].css'.
    'css file' => 'its.wysiwyg.css',
  );

  $plugins['its_locNote'] = array(
    // The plugin's title; defaulting to its internal name ('its').
    'title' => t('MultilingualWeb-LT Localization Note'),
    // The (vendor) homepage of this plugin; defaults to ''.
    'vendor url' => 'http://drupal.org/project/its',
    // The button image filename; defaults to '[plugin-name].png'.
    'icon file' => 'locNote.png',
    // The button title to display on hover.
    'icon title' => t('Add a localization note for the translator. (localization note)'),
    // An array of settings for this button. Required, but API is still in flux.
    'settings' => array(
      'validAttributes' => array('its-loc-note', 'its-loc-note-type'),
    ),
    'buttons' => array('its_locNote' => t('localization note')),
    'load' => TRUE,
    'internal' => FALSE,
    // The path to the button's icon; defaults to
    // '/[path-to-module]/[plugins-directory]/[plugin-name]/images'.
    'icon path' => drupal_get_path('module', 'its') . '/plugins/images',
    // An alternative path to the integration JavaScript; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'js path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration JavaScript; defaults to
    // '[plugin-name].js'.
    'js file' => 'its.wysiwyg.js',
    // An alternative path to the integration stylesheet; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'css path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration stylesheet; defaults to
    // '[plugin-name].css'.
    'css file' => 'its.wysiwyg.css',
  );

  $tmp = language_list();
  $language_list = array();
  foreach ($tmp as $lang) {
    $language_list[$lang->language] = $lang->name;
  }

  $plugins['its_lang'] = array(
    // The plugin's title; defaulting to its internal name ('its').
    'title' => t('MultilingualWeb-LT Language'),
    // The (vendor) homepage of this plugin; defaults to ''.
    'vendor url' => 'http://drupal.org/project/its',
    // The button image filename; defaults to '[plugin-name].png'.
    'icon file' => 'lang.png',
    // The button title to display on hover.
    'icon title' => t('Set the language of the marked content. (lang)'),
    // An array of settings for this button. Required, but API is still in flux.
    'settings' => array(
      'validAttributes' => array('lang'),
      'enabledLanguages' => $language_list,
    ),
    'buttons' => array('its_lang' => t('language')),
    'load' => TRUE,
    'internal' => FALSE,
    // The path to the button's icon; defaults to
    // '/[path-to-module]/[plugins-directory]/[plugin-name]/images'.
    'icon path' => drupal_get_path('module', 'its') . '/plugins/images',
    // An alternative path to the integration JavaScript; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'js path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration JavaScript; defaults to
    // '[plugin-name].js'.
    'js file' => 'its.wysiwyg.js',
    // An alternative path to the integration stylesheet; defaults to
    // '[path-to-module]/[plugins-directory]/[plugin-name]'.
    'css path' => drupal_get_path('module', 'its') . '/plugins',
    // An alternative filename of the integration stylesheet; defaults to
    // '[plugin-name].css'.
    'css file' => 'its.wysiwyg.css',
  );

  drupal_add_library('system', 'ui.dialog');
  if (($library = libraries_load('jquery.its-parser')) && empty($library['loaded'])) {
    drupal_set_message(t('jQuery ITS-Parser library couldn\'t be found. ITS Plugins for WYSIWYG editor will not work.'));
  }

  return $plugins;
}
