(function ($) {

  $.parseITS();

  Drupal.its = {

    /**
     * Return whether the passed node belongs to this plugin.
     *
     * @param node
     *   The currently focused DOM element in the editor content.
     * @param valid_attributes
     *   Array with valid attributes
     */
    isNode:function (node, valid_attributes) {
      var hasAtt = false;
      var el = $(node);
      $.each(valid_attributes, function (key, value) {
        if (el.is('[' + value + ']')) {
          hasAtt = true;
        }
      });
      return hasAtt;
    },

    /**
     * Add its tags to a content.
     * @param content
     * @param tags {html_name: value}
     * @param className
     * @return {*}
     */
    addTags:function (content, tags, className) {
      content = $('<div>' + content + '</div>');

      var item = $(content[0].childNodes[0]);
      $.each(tags, function (html_name, value) {
        if (value !== '') {
          item.attr(html_name, value);
        }
      });

      if (className) {
        item.addClass(className);
      }

      return content.html();
    },

    insertTags:function (data, settings, instanceId, values) {
      if (data.content != '') {
        var content = '<span>' + data.content + '</span>';
        content = this.addTags(content, values);
        Drupal.wysiwyg.instances[instanceId].insert(' ' + content);
      }
      else {
        this.addTags(data.node, values);
      }
    },

    /**
     * Remove tags from content
     * @param data {} object from wysiwyg invoke
     * @param tags [] to remove
     */
    removeTags:function (data, tags) {
      if (data.format == 'html' && this.isNode(data.node, tags)) {
        var node = $(data.node);
        $.each(tags, function (key, tag_name) {
          node.removeAttr(tag_name);
        });
        console.log(data.node.attributes.length, data.node.tagName);
        if (data.node.attributes.length == 0 && data.node.tagName.toLowerCase() == 'span') {
          node.before(data.node.innerHTML);
          node.remove();
        }
      }

    },

    /**
     * Common attach function.
     * Adds the given class to the given selector and returns the html content
     * @param content
     * @param selector
     * @param className
     * @return string
     */
    attach:function (content, selector, className) {
      var $content = $('<div>' + content + '</div>');
      $(selector, $content).addClass(className);
      return $content.html()
    },

    /**
     * Common detach function.
     * Removes the class from the given classname and removes the class, if empty
     * @param content
     * @param selector
     * @param className
     * @return string
     */
    detach:function (content, selector, className) {
      var $content = $('<div>' + content + '</div>');
      var items = $("." + className, $content);
      items.removeClass(className);
      for (var i = 0; i < items.length; i++) {
        var item = $(items[i]);
        if (!item.attr('alt')) {
          item.removeAttr('class');
        }
      }
      return $content.html();
    }

  };

  /**
   * Wysiwyg plugin button implementation for MLW-LT disambiguation plugin.
   */
  Drupal.wysiwyg.plugins.its_lexicalizes = {

    modal:null,

    isNode:function (node) {
      return Drupal.its.isNode(node, Drupal.settings.wysiwyg.plugins.drupal.its_lexicalizes.validAttributes);
    },

    invoke:function (data, settings, instanceId) {
      this.openModal(data, settings, instanceId);
    },

    openModal:function (data, settings, instanceId) {
      var defaults = {};
      var edit = false;
      if (data.format == 'html' && this.isNode(data.node)) {
        var node = $(data.node);
        defaults = {
          ident_ref:node.attr('its-disambig-ident-ref'),
          class_ref:node.attr('its-disambig-class-ref'),
          granularity:node.attr('its-disambig-granularity')
        };
        edit = true;
        $.each(defaults, function (key, value) {
          if (typeof value == "undefined") {
            defaults[key] = ""
          }
        });
      }
      else {
        defaults = {
          ident_ref:'http://dbpedia.org/resource/Dublin',
          class_ref:'http://nerd.eurecom.fr/ontology#Place',
          granularity:'entity'
        };
      }

      if (this.modal == null) {
        this.modal = $('<div id="wysiywg-mlwlt-lexicalizes" title="Disambiguation">' +
          '<label for="wysiywg-mlwlt-lexicalizes-ident-ref">Disambiguation Target</label><input id="wysiywg-mlwlt-lexicalizes-ident-ref" type="text" value="" style="width: 400px">' +
          '<label for="wysiywg-mlwlt-lexicalizes-class-ref">Class/Type</label><input id="wysiywg-mlwlt-lexicalizes-class-ref" type="text" value="" style="width: 400px">' +
          '<label for="wysiywg-mlwlt-lexicalizes-granularity">Type</label><select id="wysiywg-mlwlt-lexicalizes-granularity"><option value="entity">Entity</option><option value="lexicalConcept">Lexical Concept</option><option value="ontologyConcept">Ontology Concept</option></select>' +
          '</div>');
        this.modal.hide();
        $('body').append(this.modal);

        this.modal.dialog({
          autoOpen:false,
          modal:true,
          width:450
        });
      }

      $('#wysiywg-mlwlt-lexicalizes-disambig-type', this.modal).val(defaults.granularity);
      $('#wysiywg-mlwlt-lexicalizes-ident-ref', this.modal).val(defaults.ident_ref);
      $('#wysiywg-mlwlt-lexicalizes-class-ref', this.modal).val(defaults.class_ref);

      var that = this;
      var buttons = {};
      if (edit) {
        buttons.edit = function () {
          that.insertTags(data, settings, instanceId);
          $(this).dialog('close');
        };
        buttons.remove = function () {
          Drupal.its.removeTags(data, Drupal.settings.wysiwyg.plugins.drupal.its_lexicalizes.validAttributes)
          $(this).dialog('close');
        };
      }
      else {
        buttons.add = function () {
          that.insertTags(data, settings, instanceId);
          $(this).dialog('close');
        };
      }
      buttons.close = function () {
        $(this).dialog('close');
      };

      this.modal.dialog('option', 'buttons', buttons);

      this.modal.dialog('open');
    },

    insertTags:function (data, settings, instanceId) {
      var values = {
        'its-disambig-ident-ref':$('#wysiywg-mlwlt-lexicalizes-ident-ref').val(),
        'its-disambig-class-ref':$('#wysiywg-mlwlt-lexicalizes-class-ref').val(),
        'its-disambig-granularity':$('#wysiywg-mlwlt-lexicalizes-granularity').val()
      };
      Drupal.its.insertTags(data, settings, instanceId, values);
    }

  };

  /**
   * Wysiwyg plugin button implementation for MLW-LT locale plugin.
   */
  Drupal.wysiwyg.plugins.its_locale = {

    modal:null,

    isNode:function (node) {
      return Drupal.its.isNode(node, Drupal.settings.wysiwyg.plugins.drupal.its_locale.validAttributes);
    },

    invoke:function (data, settings, instanceId) {
      this.openModal(data, settings, instanceId);
    },

    openModal:function (data, settings, instanceId) {
      if (this.modal == null) {
        this.modal = $('<div id="wysiywg-mlwlt-locale" title="Locale Filter">' +
          '<label for="wysiywg-mlwlt-locale-list">Only for Languages: </label><input id="wysiywg-mlwlt-locale-list" type="text" value="">' +
          '<div>comma sperated list of languages<br/>wildcard (*) allowed<br/> examples: de-DE, en-*, *-CA, * or empty</div>' +
          '</div>');
        this.modal.hide();
        $('body').append(this.modal);
        this.modal.dialog({
          autoOpen:false,
          modal:true
        });
      }

      var buttons = {};
      var that = this;

      if (data.format == 'html' && this.isNode(data.node)) {
        $("wysiywg-mlwlt-locale-list", this.modal).val($(data.node).attr('its-locale-filter-list'));
        buttons.edit = function () {
          that.insertTags(data, settings, instanceId);
          $(this).dialog('close');
        };
        buttons.remove = function () {
          Drupal.its.removeTags(data, Drupal.settings.wysiwyg.plugins.drupal.its_locale.validAttributes);
          $(this).dialog('close');
        };
      }
      else {
        buttons.add = function () {
          that.insertTags(data, settings, instanceId);
          $(this).dialog('close');
        };
      }
      buttons.close = function () {
        $(this).dialog('close');
      };

      this.modal.dialog('option', 'buttons', buttons);
      this.modal.dialog('open');
    },

    insertTags:function (data, settings, instanceId) {
      var value = {'its-locale-filter-list':$('#wysiywg-mlwlt-locale-list').val()};
      Drupal.its.insertTags(data, settings, instanceId, value);
    }

  };

  /**
   * Wysiwyg plugin button implementation for MLW-LT locNote plugin.
   */
  Drupal.wysiwyg.plugins.its_locNote = {

    modal:null,

    isNode:function (node) {
      var value = $(node).getITSData();
      return value && value.locNote;
    },

    invoke:function (data, settings, instanceId) {
      this.openModal(data, settings, instanceId);
    },

    openModal:function (data, settings, instanceId) {
      var defaults = {};
      var edit = false;
      if (data.format == 'html' && this.isNode(data.node)) {
        var locNote = $(data.node).getITSData();
        defaults = {
          note:locNote.locNote,
          type:locNote.locNoteType
        };
        edit = true;
        $.each(defaults, function (key, value) {
          if (typeof value == "undefined") {
            defaults[key] = ""
          }
        });
      }
      else {
        defaults = {
          note:'',
          type:'description'
        };
      }

      if (this.modal == null) {
        this.modal = $('<div id="wysiywg-mlwlt-loc-note" title="Localization Note">' +
          '<label for="wysiywg-mlwlt-loc-note-type">Type</label><select id="wysiywg-mlwlt-loc-note-type"><option value="description">description</option><option value="alert">alert</option></select>' +
          '<label for="wysiywg-mlwlt-loc-note-text">List</label><textarea id="wysiywg-mlwlt-loc-note-text"></textarea>' +
          '</div>');
        this.modal.hide();
        $('body').append(this.modal);
        this.modal.dialog({
          autoOpen:false,
          modal:true
        });
      }

      $('#wysiywg-mlwlt-loc-note-type', this.modal).val(defaults.type);
      $('#wysiywg-mlwlt-loc-note-text', this.modal).val(defaults.note);

      var buttons = {};
      var that = this;
      if (edit) {
        buttons.edit = function () {
          that.insertTags(data, settings, instanceId);
          $(this).dialog('close');
        };
        buttons.remove = function () {
          Drupal.its.removeTags(data, Drupal.settings.wysiwyg.plugins.drupal.its_locNote.validAttributes);
          $(this).dialog('close');
        };
      }
      else {
        buttons.add = function () {
          that.insertTags(data, settings, instanceId);
          $(this).dialog('close');
        };
      }
      buttons.close = function () {
        $(this).dialog('close');
      };
      this.modal.dialog('option', 'buttons', buttons);
      this.modal.dialog('open');
    },

    insertTags:function (data, settings, instanceId) {
      var values = {
        'its-loc-note-type':$('#wysiywg-mlwlt-loc-note-type').val(),
        'its-loc-note':$('#wysiywg-mlwlt-loc-note-text').val()
      };
      Drupal.its.insertTags(data, settings, instanceId, values);
    }

  };

  /**
   * Wysiwyg plugin button implementation for MLW-LT translate plugin.
   */
  Drupal.wysiwyg.plugins.its_translate = {

    isNode:function (node) {
      var value = $(node).getITSData();
      return value && !value.translate;
    },

    invoke:function (data, settings, instanceId) {
      var insert = false;
      // Generate HTML markup.
      if (data.format == 'html') {
        // Prevent duplications
        if (this.isNode(data.node)) {
          Drupal.its.removeTags(data, Drupal.settings.wysiwyg.plugins.drupal.its_translate.validAttributes);
        }
        else {
          insert = true;
        }
      }
      // Generate plain text.
      else {
        insert = true;
      }
      // Insert new content into the editor.
      if (insert) {
        var value = 'no';
        // Chrome supports translate already and needs true/false, when setting this instead of yes/no
        var chrome = navigator.userAgent.match(/Chrome\/(\d+)/);
        if (chrome && chrome[1] > 22) {
          value = false;
        }
        console.log(value, chrome);
        if (data.content != '') {
          var content = '<span>' + data.content + '</span>';
          content = Drupal.its.addTags(content, {translate:value}, 'its-translate');
          Drupal.wysiwyg.instances[instanceId].insert(content);
        }
        else {
          Drupal.its.addTags(data.node, {translate:value}, 'its-translate');
        }
      }
    },

    attach:function (content, settings, instanceId) {
      return Drupal.its.attach(content, ':translate(no)', 'its-translate');
    },

    detach:function (content, settings, instanceId) {
      return Drupal.its.detach(content, ':translate(no)', 'its-translate');
    }

  };

  /**
   * Wysiwyg plugin button implementation for MLW-LT translate plugin.
   */
  Drupal.wysiwyg.plugins.its_lang = {

    modal:null,

    isNode:function (node) {
      return Drupal.its.isNode(node, Drupal.settings.wysiwyg.plugins.drupal.its_lang.validAttributes);
    },

    invoke:function (data, settings, instanceId) {
      this.openModal(data, settings, instanceId);
    },

    openModal:function (data, settings, instanceId) {
      var defaults = {};
      var edit = false;
      if (data.format == 'html' && this.isNode(data.node)) {
        var node = $(data.node);
        defaults = {
          lang:node.attr('lang')
        };
        edit = true;
        $.each(defaults, function (key, value) {
          if (typeof value == "undefined") {
            defaults[key] = ""
          }
        });
      }
      else {
        defaults = {
          lang:''
        };
      }

      var that = this;
      if (this.modal == null) {
        this.modal = $('<div id="wysiywg-mlwlt-lang" title="Language">' +
          '<label for="wysiywg-mlwlt-lang-lang">Type</label><select id="wysiywg-mlwlt-lang-lang"></select>' +
          '</div>');
        $.each(Drupal.settings.wysiwyg.plugins.drupal.its_lang.enabledLanguages, function (key, val) {
          $('#wysiywg-mlwlt-lang-lang', that.modal).append('<option value="' + key + '">' + val + '</option>');
        });
        this.modal.hide();
        $('body').append(this.modal);
        this.modal.dialog({
          autoOpen:false,
          modal:true
        });
      }

      $('#wysiywg-mlwlt-lang-lang', this.modal).val(defaults.lang);

      var buttons = {};
      if (edit) {
        buttons.edit = function () {
          that.insertTags(data, settings, instanceId);
          $(this).dialog('close');
        };
        buttons.remove = function () {
          Drupal.its.removeTags(data, Drupal.settings.wysiwyg.plugins.drupal.its_lang.validAttributes);
          $(this).dialog('close');
        };
      }
      else {
        buttons.add = function () {
          that.insertTags(data, settings, instanceId);
          $(this).dialog('close');
        };
      }
      buttons.close = function () {
        $(this).dialog('close');
      };
      this.modal.dialog('option', 'buttons', buttons);
      this.modal.dialog('open');
    },

    insertTags:function (data, settings, instanceId) {
      var values = {
        'lang':$('#wysiywg-mlwlt-lang-lang').val()
      };
      Drupal.its.insertTags(data, settings, instanceId, values);
    }

  };

}(jQuery));
